
-- n: u
-- bit: u - 0-based index, starting at LSB
--> bool  - true/false if bit is 1/0
local function getbit(n, bit)
	return 1 == (n/2^bit)%2 - (n/2^bit)%1
end


-- rule: u8 - Wolfram Code
--> { [bool]={ [bool]={ [bool]=bool } } } - rule lookup, starting at left
local function wolframrules(rule)
	return {
		[true]  = {
			[true]  = {
				[true]  = getbit(rule, 7),
				[false] = getbit(rule, 6),
			},
			[false] = {
				[true]  = getbit(rule, 5),
				[false] = getbit(rule, 4),
			},
		},
		[false] = {
			[true]  = {
				[true]  = getbit(rule, 3),
				[false] = getbit(rule, 2),
			},
			[false] = {
				[true]  = getbit(rule, 1),
				[false] = getbit(rule, 0),
			},
		},
	}
end


-- rule: u8             - Wolfram Code
-- state: { bool, ... } - World state
--> { bool, ... }       - Next state, same length as input
return function(rule, state)
	local newstate = {}
	local rules = wolframrules(rule)
	for i = 1,#state do
		newstate[i] = rules
			[state[i-1] or false]
			[state[i]]
			[state[i+1] or false]
	end
	return newstate
end
